@@ -1,11 +1,15 @@
 package Data::DPath::Filters;
 
-use 5.010;
 use strict;
 use warnings;
 
 use Data::Dumper;
 use Scalar::Util;
+use constant {
+              HASH   => 'HASH',
+              ARRAY  => 'ARRAY',
+              SCALAR => 'SCALAR',
+      };
 
 our $idx;
 our $p;   # current point
@@ -16,57 +20,61 @@ sub affe {
 
 sub idx { $idx }
 
-sub size
+sub size()
 {
         no warnings 'uninitialized';
-        return scalar @$_      if Scalar::Util::reftype $_  eq 'ARRAY';
-        return scalar keys %$_ if Scalar::Util::reftype $_  eq 'HASH';
-        return  1              if Scalar::Util::reftype \$_ eq 'SCALAR';
+
+        # speed optimization: first try faster ref, then reftype
+        # ref
+        return scalar @$_      if (defined $_ and ref $_  eq ARRAY);
+        return scalar keys %$_ if (defined $_ and ref $_  eq HASH);
+        return  1              if (defined $_ and ref \$_ eq SCALAR);
+        # reftype
+        return scalar @$_      if (defined $_ and Scalar::Util::reftype $_  eq ARRAY);
+        return scalar keys %$_ if (defined $_ and Scalar::Util::reftype $_  eq HASH);
+        return  1              if (defined $_ and Scalar::Util::reftype \$_ eq SCALAR);
+        # else
         return -1;
 }
 
-sub key
+sub key()
 {
         no warnings 'uninitialized';
-        my $attrs = $p->attrs // {};
+        my $attrs = defined $p->attrs ? $p->attrs : {};
         return $attrs->{key};
 }
 
-sub value
+sub value()
 {
-        #print STDERR "*** value ", (keys %$_)[0], " ", Dumper($_ ? $_ : "UNDEF");
         no warnings 'uninitialized';
-        return (values %$_)[0] if (defined $_ and Scalar::Util::reftype  $_  eq 'HASH');
-        return $_              if (defined $_ and Scalar::Util::reftype \$_  eq 'SCALAR');
+
+        # optimization: first try faster ref, then reftype
+        # ref
+        return (values %$_)[0] if (defined $_ and ref  $_ eq HASH);
+        return $_              if (defined $_ and ref \$_ eq SCALAR);
+        # reftype
+        return (values %$_)[0] if (defined $_ and Scalar::Util::reftype  $_ eq HASH);
+        return $_              if (defined $_ and Scalar::Util::reftype \$_ eq SCALAR);
+        # else
         return undef;
 }
 
-sub isa {
+sub isa($) {
         my ($classname) = @_;
 
+        no warnings 'uninitialized';
         #print STDERR "*** value ", Dumper($_ ? $_ : "UNDEF");
-        return $_->isa($classname) if (defined $_ and Scalar::Util::blessed $_);
+        return $_->isa($classname) if Scalar::Util::blessed $_;
         return undef;
 }
 
-sub reftype {
-        my ($refname) = @_;
-
-        #print STDERR "*** value ", Dumper($_ ? $_ : "UNDEF");
-        return Scalar::Util::reftype($_) if not $refname;
-        return (Scalar::Util::reftype($_) eq $refname);
+sub reftype() {
+        return Scalar::Util::reftype($_);
 }
 
-# sub parent, Eltern-Knoten liefern
-# nextchild, von parent und mir selbst
-# previous child
-# "." als aktueller Knoten, kind of "no-op", daran aber Filter verknüpfbar, löst //.[filter] und /.[filter]
-
-# IDEA: functions that return always true, but track stack of values, eg. last taken index
-#
-#    //AAA/*[ _push_idx ]/CCC[ condition ]/../../*[ idx == pop_idx + 1]/
-#
-# This would take a way down to a filtered CCC, then back again and take the next neighbor.
+sub is_reftype($) {
+        return (Scalar::Util::reftype($_) eq shift);
+}
 
 1;
 
@@ -122,12 +130,16 @@ class.
 
 Frontend to Scalar::Util::reftype.
 
-If argument given it checks whether reftype($_) equals the argument
-and returns true/false.
+Returns Scalar::Util::reftype of current element $_. With this you can
+do comparison by yourself with C<eq>, C<=~>, C<~~> or whatever in
+filter expressions.
 
-If no argument is given it returns reftype of current element $_ and
-you can do comparison by yourself with C<eq>, C<=~>, C<~~> or
-whatever.
+=head2 is_reftype($EXPECTED_TYPE)
+
+Frontend to Scalar::Util::reftype.
+
+Checks whether Scalar::Util::reftype of current element $_ equals the
+provided argument $EXPECTED_TYPE and returns true/false.
 
 =head1 AUTHOR
 
@@ -141,3 +153,15 @@ This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
+
+# sub parent, Eltern-Knoten liefern
+# nextchild, von parent und mir selbst
+# previous child
+# "." als aktueller Knoten, kind of "no-op", daran aber Filter verknüpfbar, löst //.[filter] und /.[filter]
+
+# IDEA: functions that return always true, but track stack of values, eg. last taken index
+#
+#    //AAA/*[ _push_idx ]/CCC[ condition ]/../../*[ idx == pop_idx + 1]/
+#
+# This would take a way down to a filtered CCC, then back again and take the next neighbor.
+
